using Demo.Domain.Models;
using Demo.Domain.UseCase;
using Microsoft.AspNetCore.Mvc;

namespace presence_api.Controllers;
[ApiController]
[Route("api/[controller]")]

public class PresenceController: ControllerBase{

    private readonly PresenceUseCase _presenceUseCase;
    private readonly ILogger<PresenceController> _logger;

    public PresenceController(PresenceUseCase presenceUseCase, ILogger<PresenceController> logger){

    _presenceUseCase = presenceUseCase;
    _logger = logger;

    }

    [HttpPost]
    public ActionResult<bool> GeneratePresence(int firstLesson, int lastLesson, int groupID, string date) {

        if (!DateOnly.TryParse(date, out var parseDate))
        {
            _logger.LogWarning("GeneratePresence: Invalid date format");
            return BadRequest("Invalid date format");
        }

        var isGenerate = _presenceUseCase.GeneratePresence(firstLesson, lastLesson, groupID, parseDate);
        return Ok(true);
    }

    [HttpGet]
    public ActionResult<PresenceResponse> GetPresence(int groupID, string start = null, string end = null, Guid userGuid = default)
    {
        PresenceResponse? presences = null;
        if (start != null && end != null)
        {
            DateOnly.TryParse(start, out DateOnly start1);
            DateOnly.TryParse(end, out DateOnly end1);
            presences = _presenceUseCase.GetPresencebyAll(groupID, start1, end1, userGuid);
        } else {
            presences = _presenceUseCase.GetPresencebyAll(groupID, null, null, userGuid);
        }
        return Ok(presences);
    }

    [HttpDelete("user")]
    public ActionResult<bool> DeletePresenceByUserGuid(Guid userGuid)
    {
        if (userGuid == Guid.Empty){
            _logger.LogWarning("DeletePresenceByUserGuid: userGuid empty");
            return BadRequest("Invalid request");
        }

        var isDeleted = _presenceUseCase.DeletePresenceByUser(userGuid);
        if (isDeleted == false){
            _logger.LogWarning("DeletePresenceByUserGuid: User not found");
            return NotFound("User not found");
        }
        return Ok(true);
    }

    [HttpDelete("group")]
    public ActionResult<bool> DeletePresenceByGroup(int groupID)
    {
        var isDeleted = _presenceUseCase.DeletePresenceByGroup(groupID);
        if (isDeleted == false){
            _logger.LogWarning("DeletePresenceByGroup: Group not found");
            return NotFound("Group not found");
        }
        return Ok(true);
    }

    [HttpDelete("datarange")]
    public ActionResult<bool> DeletePresenceByDateRange(string startDate, string endDate)
    {
        if (!DateOnly.TryParse(startDate, out var parsedStartDate) || 
            !DateOnly.TryParse(endDate, out var parsedEndDate))
        {
            _logger.LogWarning("DeletePresenceByDateRange: Invalid date format");
            return BadRequest("Invalid date format");
        }

        var isDeleted = _presenceUseCase.DeletePresenceByDateRange(parsedStartDate, parsedEndDate);
        if (isDeleted == false){
            _logger.LogWarning("DeletePresenceByDateRange: Presence not found");
            return NotFound("Presence not found");
        }

        return Ok(true);
    }

    [HttpPatch]
    public ActionResult<bool> UpdatePresence(int firstLesson, int lastLesson, string date, Guid UserGuid)
    {
        if (!DateOnly.TryParse(date, out var parseDate))
        {
            _logger.LogWarning("UpdatePresence: Invalid date format");
            return BadRequest("Invalid date format");
        }

        var isUpdated = _presenceUseCase.IsAttedance(firstLesson, lastLesson, parseDate, UserGuid);
        if (isUpdated == false){
            _logger.LogWarning("UpdatePresence: Presence not found");
            return NotFound("Presence not found");
        }

        return Ok(true);
    }
}